@@ -1,8 +1,18 @@
 
+.drop <- function(x) {
+    if (inherits(x, "Matrix")) x <- x@x
+    drop(x)
+}
+
+.pmax <- function(x, y) {
+    y[y < x] <- x
+    return(y)
+}
+
 .xmb <- function(X, beta) {
     if (is.matrix(beta)) {
         stopifnot(NROW(beta) == NROW(X))
-        return(rowSums(as(X, "matrix") * beta))
+        return(rowSums(X * beta))
     }
     return(X %*% beta)
 }
@@ -10,26 +20,26 @@
 ### if (finite) fun(X %*% beta + offset) * Xmult else value
 .dealinf <- function(X, beta, offset, fun, value, Xmult = FALSE) {
     if (is.null(X)) return(value)
-    OK <- is.finite(X[,1])
+    lpr <- .drop(offset + .xmb(X, beta))
+    OK <- is.finite(lpr)
     if (all(OK)) {
-        ret <- drop(fun(offset + .xmb(X, beta)))
+        ret <- fun(lpr)
     } else {
-        if (all(!OK)) return(rep(value, nrow(X)))
-        if (is.matrix(beta)) 
-            beta <- beta[OK,,drop = FALSE]
-        tmp <- .xmb(X[OK,], beta)
-        ret <- numeric(nrow(X))
-        ret[OK] <- fun(offset[OK] + tmp)
-        ret[!OK] <- value
-        X[!OK,] <- 0
+        ret <- rep(value, length(lpr))
+        if (all(!OK)) return(ret)
+        ret[OK] <- fun(lpr[OK])
     }
-    if (Xmult)
+    if (Xmult) {
+        ### this is costly, can we avoid this?
+        if (!all(OK))
+            X[!OK,] <- 0
         ret <- ret * X
+    }
     return(ret)
 }
 
 .log <- function(x) {
-    return(log(pmax(.Machine$double.eps, x)))
+    return(log(.pmax(.Machine$double.eps, x)))
     pos <- (x > .Machine$double.eps)
     if (all(pos)) return(log(x))
 #    mx <- min(x)
@@ -58,7 +68,7 @@
 ..mlt_score_interval <- function(d, mml, mmr, offset = 0, beta, Xmult = TRUE)
     (.dealinf(mmr, beta, offset, d$d, 0, Xmult = Xmult) -
      .dealinf(mml, beta, offset, d$d, 0, Xmult = Xmult)) / 
-    pmax(.Machine$double.eps^(1/3), (.dealinf(mmr, beta, offset, d$p, 1) - 
+    .pmax(.Machine$double.eps^(1/3), (.dealinf(mmr, beta, offset, d$p, 1) - 
      .dealinf(mml, beta, offset, d$p, 0)))
 
 .mlt_score_interval <- function(d, mml, mmr, offset = 0, mmtrunc = NULL) {
